/*******************************************************************************************
 * This header file contains the constants, structure definitions, and the library procedure 
 * interfaces.  
 *
 * Note that usually we don't execute transitions on state images.  
 * This is because that there could be synchronizations among transitions.  
 * RED compiles a table called synchronous transition table in which 
 * each entry corresponds to a minimal combination of synchronizing transitions.  
 * RED also compiles another transition bulk CRD+MDD for the bulk evaluation of 
 * pre or post images for a set of complex synchronization of transitions.  
 * Thus unless for curioristy, users should use synchronous transition table 
 * to calculate the pre or post image.  
 * 
 * The synchronous transitions are stored in array SYNC_XTION[..] whose entry count 
 * is stored in SYNC_XTION_COUNT.  
 * If all your synchronizations are pairwise, then this should store all the synchronizous 
 * transitions you need, including those transitions without synchronization.  
 * The synchronous transitions of those complex synchronizations are stored in 
 * CRD+MDD RED_complex_sync_xtions().  
 * The relevant procedures for computing post and pre-images are as follows. 
 *   RED_sync_xtion_specific_fwd(d, sxi) : post image of a particular synchronous transiton indexed sxi
 *   RED_sync_xtion_specific_bck(d, sxi) : pre-image of a particular synchronous transiton indexed sxi
 *   RED_sync_xtion_iterative_fwd(d) : post image of all simple synchronous transitons stored in SYNC_XTION[]
 *   RED_sync_xtion_iterative_bck(d) : pre-image of all simple synchronous transitons stored in SYNC_XTION[]
 *   RED_sync_xtion_bulk_fwd(d) : post image of all complex synchronous transitons stored 
 *				  in RED_complex_sync_xtions().  
 *   RED_sync_xtion_bulk_bck(d) : pre-image of all complex synchronous transitons stored 
 *				  in RED_complex_sync_xtions(). 
 *   RED_sync_xtion_fwd(d) : post image of all synchronous transitions. 
 *   RED_sync_xtion_bck(d) : post image of all synchronous transitions. 
 */
/******************************************************************************
 * The plan for the interface.  
 *	All the constants are to be copied from the redbasics.h, ... 
 * Thus it is not OK to revise the constants in this file. 
 * You have to modify them in the corresponding files and then cut and paste here. 
 */
#ifndef _REDLIB_H
#define _REDLIB_H

typedef struct red_type *redgram; 

/****** redbasics.h ******************************************************************
 * The following constants are defined in redbasics.h 
 */ 


/* The following flags are used to set options in controlling the operation 
 * of the red engine. 
    The option can be set with the following procedure. 
    void	RED_option_set(int opt, int value)
    opt is a macro constant like RED_OPT_... declared in the following.  
    value is the value of the option to be set. 
    For example, RED_option_set(RED_OPT_ABSTRACTION, FLAG_OVER_APPROX) sets 
    the abstraction to over-approximation.  
    
    We can also set the flag with bit-values through the following procedure.   
    void	RED_flag_set(int opt)
    Here opt is the macro constant like RED_OPT_... used in RED_option_set(opt, ...).  
    
    We can also clear the flag with bit-values through the following procedure.   
    void	RED_flag_clear(int opt)
    Here opt is the macro constant like RED_OPT_... used in RED_option_set(opt, ...).  
    
    We can also check the flag value with the following procedure. 
    int	RED_flag_check(int opt) 
    Here opt is the macro constant like RED_OPT_... used in RED_option_set(opt, ...).  	
 */ 

/* MASK values for GLOBAL_STATUS[0]  */





/********************************************************************************
 * The following are constants used to interface redlib.a
 */


/* Some string used for easy construction of the model. 
 */ 
#define RED_MODE           "RED_MODE"
#define RED_CONDITION_TRUE "true"
#define RED_CONDITION_FALSE "false"
#define RED_OO             (INT_MAX)
#define RED_P              "P"
#define RED_Q              "Q"
#define RED_R              "R"
#define RED_S              "S"

/* 
typedef struct red_type RED_TYPE;  //struct red_type in redbop.h 
typedef struct xtion_type XTION_TYPE; //struct xtion_type in red.h 
typedef struct action_type ACTION_TYPE; //struct action_type in red.h 
*/ 

/* Type constants for the declaration of variables in model construction.  
 * Used for the type in RED_begin_variable_declaration(scope, type)  
 */
#define	RED_VAR_TYPE		1

#define RED_TYPE_SYNCHRONIZER   	1002
#define RED_TYPE_DISCRETE       	1003
#define RED_TYPE_DENSE          	1004
#define RED_TYPE_CLOCK          	1005
#define RED_TYPE_POINTER        	1006
#define RED_TYPE_PARAMETER      	1007
#define RED_TYPE_MTERMINAL		1008
#define RED_TYPE_BOOLEAN		1009
#define RED_TYPE_CRD	1010

#define	RED_VAR_SCOPE				2

#define RED_SCOPE_GLOBAL        		1010
#define RED_SCOPE_LOCAL         		1011
 
#define	RED_TASK_TYPE				1020

#define RED_TASK_REDLIB_SESSION			1
#define RED_TASK_SAFETY				2
#define RED_TASK_RISK				3 
#define RED_TASK_GOAL				4 
#define RED_TASK_ZENO				5 
#define RED_TASK_DEADLOCK			6 
#define RED_TASK_MODEL_CHECK			7 
#define RED_TASK_SIMULATE			8 
#define RED_TASK_BRANCH_SIM_CHECK		9 
#define RED_TASK_BRANCH_BISIM_CHECK		10 

#define	RED_VAR_PRIMED				1030 
#define	RED_VAR_PRIMED				1030 
#define RED_VAR_PROC				1040 
#define RED_VAR_LB				1050 
#define RED_VAR_UB				1060 
#define RED_VAR_CLOCK1				1070 
#define RED_VAR_CLOCK2				1080 
#define RED_VAR_SYSGEN 				1090


/* Used for the op in RED_formulus_ineq(lhs_name, op, rhs_name)
 */
#define RED_OP_LESS             ARITH_LESS
#define RED_OP_LEQ              ARITH_LEQ
#define RED_OP_EQ               ARITH_EQ
#define RED_OP_NEQ              ARITH_NEQ
#define RED_OP_GEQ              ARITH_GEQ
#define RED_OP_GREATER          ARITH_GREATER


#define RED_XTION_SYNC_COUNT	01 
#define RED_XTION_SRC_MODE	02 
#define RED_XTION_DST_MODE	03 
#define RED_XTION_PROCESS_COUNT	04 



#define RED_MODE_XTION_COUNT	100 
#define RED_MODE_URGENT		101 
#define RED_MODE_PROCESS_COUNT	102 
#define RED_MODE_RATE_LB_NUM	200
#define RED_MODE_RATE_LB_DEN	201 
#define RED_MODE_RATE_UB_NUM	202 
#define RED_MODE_RATE_UB_DEN	203 
 

#define RED_SYSTEM_UNTIMED	100
#define RED_SYSTEM_TIMED	101
#define RED_SYSTEM_HYBRID	102
#define RED_SYSTEM_UNKNOWN	103
 
#define RED_ARITH_ADD      	ARITH_ADD
#define RED_ARITH_MINUS    	ARITH_MINUS
#define RED_ARITH_MULTIPLY 	ARITH_MULTIPLY
#define RED_ARITH_DIVIDE   	ARITH_DIVIDE
#define	RED_ARITH_MODULO	ARITH_MODULO
#define RED_ARITH_MIN   	ARITH_MIN
#define RED_ARITH_MAX   	ARITH_MAX



#define	RED_COVERAGE_ACM	(0x01)
#define	RED_COVERAGE_DCM	(0x02)
#define RED_COVERAGE_TCM	(0x04)
#define	RED_COVERAGE_RCM	(0x08) 
#define	RED_COVERAGE_DTCM	(0x10)

#define	RED_ALL_ROLES		(0x07) 
#define	RED_SIM_ROLES		(0x07) 
#define	RED_BISIM_ROLES		(0x07) 
#define	RED_SIM_MODL		(0x01) 
#define	RED_BISIM_MODL		(0x01) 
#define	RED_SIM_SPEC		(0x02) 
#define	RED_BISIM_SPEC		(0x02) 
#define RED_SIM_ENVR		(0x04) 
#define RED_BISIM_ENVR		(0x04) 

#define RED_FULL_REACHABILITY		(0x01) 
#define	RED_NO_FULL_REACHABILITY	(0x00) 

#define RED_NO_REACHABILITY_DEPTH_BOUND	-1 

#define RED_COUNTER_EXAMPLE	(0x01) 
#define	RED_NO_COUNTER_EXAMPLE	(0x00) 

#define RED_TIME_PROGRESS	(0x01) 
#define	RED_NO_TIME_PROGRESS	(0x00) 

#define	RED_NORM_ZONE_NONE			(0x00)
#define RED_NORM_ZONE_MAGNITUDE_REDUCED		(0x01)
#define	RED_NORM_ZONE_CLOSURE			(0x02)
#define RED_NORM_ZONE_MAGNITUDE_ONE_RIPPLE	(0x03) 

#define RED_NORM_HYBRID_2REDUNDANCY_ELIMINATION_DOWNWARD	(0x00010000)
#define RED_NORM_HYBRID_3REDUNDANCY_ELIMINATION_DOWNWARD	(0x00020000)
#define RED_NORM_HYBRID_4REDUNDANCY_ELIMINATION_DOWNWARD	(0x00040000)
#define RED_NORM_HYBRID_2REDUNDANCY_ELIMINATION_LOOKAHEAD	(0x00080000)
#define RED_NORM_HYBRID_PROOF_OBLIGATIONS			(0x00100000)

#define	RED_NO_ACTION_APPROX		(0x00)
#define	RED_ACTION_APPROX_NOXTIVE	(0x01)
#define	RED_ACTION_APPROX_UNTIMED	(0x02)

#define	RED_NO_REDUCTION_INACTIVE	(0x00)
#define	RED_REDUCTION_INACTIVE_NOXTIVE	(0x01)
#define	RED_REDUCTION_INACTIVE		(0x02)

#define	RED_PLAIN_NONZENO	(0x02)
#define RED_APPROX_NONZENO	(0x01)
#define RED_ZENO_TRACES_OK	(0x00) 

#define RED_PARAMETRIC_ANALYSIS		(0x01) 
#define	RED_NO_PARAMETRIC_ANALYSIS	(0x00) 

#define	RED_GARBAGE_SILENT	0
#define	RED_GARBAGE_REPORT	1


#define	RED_MASK_APPROX			(0x3000000)
#define	RED_NOAPPROX			(0x0000000)
#define	RED_UAPPROX			(0x1000000) 
#define	RED_OAPPROX			(0x2000000)

#define RED_MASK_OAPPROX_STRATEGY	(0xF0000) 

#define	RED_OAPPROX_STRATEGY_GAME	(0x10000)

#define RED_MASK_OAPPROX_SPEC_GAME	(0xF000F) 
#define	RED_NOAPPROX_SPEC_GAME		(0x00000) 
#define RED_OAPPROX_SPEC_GAME_DIAG_MAG	(0x10001)
#define RED_OAPPROX_SPEC_GAME_DIAGONAL	(0x10002)
#define RED_OAPPROX_SPEC_GAME_MAGNITUDE	(0x10003)
#define RED_OAPPROX_SPEC_GAME_UNTIMED	(0x10004)
#define RED_OAPPROX_SPEC_GAME_MODE_ONLY	(0x10005)
#define RED_OAPPROX_SPEC_GAME_NONE	(0x10006)

#define RED_MASK_OAPPROX_MODL_GAME	(0xF00F0) 
#define	RED_NOAPPROX_MODL_GAME		(0x00000) 
#define RED_OAPPROX_MODL_GAME_DIAG_MAG	(0x10010)
#define RED_OAPPROX_MODL_GAME_DIAGONAL	(0x10020)
#define RED_OAPPROX_MODL_GAME_MAGNITUDE	(0x10030)
#define RED_OAPPROX_MODL_GAME_UNTIMED	(0x10040)
#define RED_OAPPROX_MODL_GAME_MODE_ONLY	(0x10050)
#define RED_OAPPROX_MODL_GAME_NONE	(0x10060)

#define RED_MASK_OAPPROX_ENVR_GAME	(0xF0F00) 
#define	RED_NOAPPROX_ENVR_GAME		(0x00000) 
#define RED_OAPPROX_ENVR_GAME_DIAG_MAG	(0x10100)
#define RED_OAPPROX_ENVR_GAME_DIAGONAL	(0x10200)
#define RED_OAPPROX_ENVR_GAME_MAGNITUDE	(0x10300)
#define RED_OAPPROX_ENVR_GAME_UNTIMED	(0x10400)
#define RED_OAPPROX_ENVR_GAME_MODE_ONLY	(0x10500)
#define RED_OAPPROX_ENVR_GAME_NONE	(0x10600)

#define RED_MASK_OAPPROX_GLOBAL_GAME		(0xFF000) 
#define RED_NOAPPROX_GLOBAL_GAME		(0x00000)
#define RED_OAPPROX_GLOBAL_GAME_DIAG_MAG	(0x11000)
#define RED_OAPPROX_GLOBAL_GAME_DIAGONAL	(0x12000)
#define RED_OAPPROX_GLOBAL_GAME_MAGNITUDE	(0x13000)
#define RED_OAPPROX_GLOBAL_GAME_UNTIMED		(0x14000)
#define RED_OAPPROX_GLOBAL_GAME_NONE		(0x16000)

#define RED_OAPPROX_GAME_DIAG_MAG	(  RED_OAPPROX_SPEC_GAME_DIAG_MAG \
					 | RED_OAPPROX_MODL_GAME_DIAG_MAG \
					 | RED_OAPPROX_ENVR_GAME_DIAG_MAG \
					 | RED_OAPPROX_GLOBAL_GAME_DIAG_MAG \
					 )
#define RED_OAPPROX_GAME_DIAGONAL	(  RED_OAPPROX_SPEC_GAME_DIAGONAL \
					 | RED_OAPPROX_MODL_GAME_DIAGONAL \
					 | RED_OAPPROX_ENVR_GAME_DIAGONAL \
					 | RED_OAPPROX_GLOBAL_GAME_DIAGONAL \
					 )
#define RED_OAPPROX_GAME_MAGNITUDE	(  RED_OAPPROX_SPEC_GAME_MAGNITUDE \
					 | RED_OAPPROX_MODL_GAME_MAGNITUDE \
					 | RED_OAPPROX_ENVR_GAME_MAGNITUDE \
					 | RED_OAPPROX_GLOBAL_GAME_MAGNITUDE \
					 )
#define RED_OAPPROX_GAME_LOCAL_MAGNITUDE	(  RED_OAPPROX_SPEC_GAME_MAGNITUDE \
					 | RED_OAPPROX_MODL_GAME_MAGNITUDE \
					 | RED_OAPPROX_ENVR_GAME_MAGNITUDE \
					 | RED_NOAPPROX_GLOBAL_GAME \
					 )
#define RED_OAPPROX_GAME_UNTIMED	(  RED_OAPPROX_SPEC_GAME_UNTIMED \
					 | RED_OAPPROX_MODL_GAME_UNTIMED \
					 | RED_OAPPROX_ENVR_GAME_UNTIMED \
					 | RED_OAPPROX_GLOBAL_GAME_UNTIMED \
					 )


/*****************************************
 * options for process-oriented symmetry reduction. 
 */
#define RED_NO_SYMMETRY		0
#define RED_SYMMETRY_ZONE	1
#define RED_SYMMETRY_DISCRETE	2
#define	RED_SYMMETRY_POINTER	11
#define RED_SYMMETRY_STATE	3

/* All BDD+CRD+HRD not in the user stack will be recycled. 
 * The following routine push a BDD+CRD+HRD into the user stack to
 * protect it from garbage collection.  
 * The returned index can be used to access the BDD+CRD+HRD with 
 * RED_user().  
 */ 
/* The stack to save CRD+MTMDD+HRD from garbage collection.  
 */ 

#define RED_NO_COVERAGE		0

#define RED_NO_PRINT		0



  
#endif 

















